@@ -1,6 +1,6 @@
 /* server.c - YP server routines.
 
-   Copyright (c) 1996, 1997, 1998, 1999 Thorsten Kukuk, Germany
+   Copyright (c) 1996, 1997, 1998, 1999, 2000 Thorsten Kukuk, Germany
    Copyright (c) 1993 Signum Support AB, Sweden
 
    This file is part of the NYS YP Server.
@@ -40,7 +40,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
-#include "yp.h"
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -48,12 +47,14 @@
 #include <sys/param.h>
 #include <errno.h>
 #include <stdarg.h>
+#include <alloca.h>
 
+#include "yp.h"
 #include "ypserv.h"
 #include "yp_msg.h"
 #include "yp_db.h"
 
-int children = 0;
+volatile int children = 0;
 int forked = 0;
 int xfr_check_port = 1;
 
@@ -248,7 +249,7 @@ static inline unsigned long
 get_dtm (const char *domain, const char *map)
 {
   struct stat sbuf;
-  char buf[strlen (domain) + strlen (map) + 3];
+  char *buf = alloca(strlen (domain) + strlen (map) + 3);
 
   sprintf (buf, "%s/%s", domain, map);
 
@@ -367,6 +368,7 @@ ypproc_match_2_svc (ypreq_key *key, struct svc_req *rqstp)
       struct hostent *he;
       if (children < MAX_CHILDREN)
 	{
+	  ++children;
 	  switch (fork ())
 	    {
 	    case 0:
@@ -378,19 +380,19 @@ ypproc_match_2_svc (ypreq_key *key, struct svc_req *rqstp)
 	      ypdb_close_all ();
 	      break;
 	    case -1:
+	      --children;
 	      yp_msg ("Cannot fork: %s\n", strerror (errno));
 	      result.stat = YP_YPERR;
 	      return &result;
 	      break;
 	    default:
-	      ++children;
 	      return NULL;
 	      break;
 	    }
 	}
       else
 	{
-	  yp_msg ("WARNING: ypserv has to many running children !\n");
+	  yp_msg ("WARNING(match): too many running children for DNS requests!\n");
 	  result.stat = YP_YPERR;
 	  return &result;
 	}
@@ -857,12 +859,19 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
     }
 #endif
 
+  ++children;
   switch (fork ())
     {
     case 0:
       {
-	char ypxfr_command[strlen (YPBINDIR) + 8];
+	char *ypxfr_command = alloca(strlen (YPBINDIR) + 8);
 	char g[30], t[30], p[30];
+	int i;
+
+	umask(0);
+	i = open("/dev/null", O_RDWR);
+	dup(i);
+	dup(i);
 
 	sprintf (ypxfr_command, "%s/ypxfr", YPBINDIR);
 	sprintf (t, "%u", xfr->transid);
@@ -881,6 +890,7 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	exit (0);
       }
     case -1:
+      --children;
       yp_msg ("Cannot fork: %s", strerror (errno));
       result.xfrstat = YPXFR_XFRERR;
     default:
@@ -1042,8 +1052,23 @@ ypproc_all_2_svc (ypreq_nokey *nokey, struct svc_req *rqstp)
   if (valid > 200)
     mangle = valid - 200;
 
+  if (children >= MAX_CHILDREN)
+    {
+      int wait = 0;
+
+      while (wait < 5)
+	{
+	  sleep (1);
+
+	  if (children < MAX_CHILDREN)
+	    break;
+	  ++wait;
+	}
+    }
+
   if (children < MAX_CHILDREN)
     {
+      ++children;
       switch (fork ())
 	{
 	case 0:
@@ -1057,17 +1082,22 @@ ypproc_all_2_svc (ypreq_nokey *nokey, struct svc_req *rqstp)
 	  ypdb_close_all ();
 	  break;
 	case -1:
-	  yp_msg ("Cannot fork: %s\n", strerror (errno));
-	  break;
+	  --children;
+	  yp_msg ("WARNING(ypproc_all_2_svc): cannot fork: %s\n",
+		  strerror (errno));
+	  result.ypresp_all_u.val.stat = YP_YPERR;
+	  return &result;
 	default:
-	  ++children;
 	  return NULL;
 	  break;
 	}
     }
   else
-    yp_msg ("WARNING: ypserv has to many running children ! We don't fork\n");
-
+    {
+      yp_msg ("WARNING(ypproc_all_2_svc): too many running children!\n");
+      result.ypresp_all_u.val.stat = YP_YPERR;
+      return &result;
+    }
 
   xdr_ypall_cb.u.encode = NULL;
   xdr_ypall_cb.u.close = NULL;
@@ -1323,7 +1353,7 @@ ypproc_order_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
 	    }
 	  else
 	    {
-	      char buf[val.dsize + 2];
+	      char *buf = alloca(val.dsize + 2);
 
 	      memcpy (buf, val.dptr, val.dsize);
 	      buf[val.dsize + 1] = '\0';
