@@ -487,7 +487,6 @@ yppush_foreach (const char *host)
   SVCXPRT *CallbackXprt;
   CLIENT *PushClient = NULL;
   struct ypreq_xfr req;
-  struct ypresp_xfr resp;
   struct timeval tv = {10, 0};
   u_int transid;
   char server[YPMAXPEER + 2];
@@ -578,15 +577,13 @@ yppush_foreach (const char *host)
 	}
 
       if (clnt_call (PushClient, YPPROC_XFR, (xdrproc_t) xdr_ypreq_xfr,
-		     (caddr_t) &req, (xdrproc_t) xdr_ypresp_xfr,
-		     (caddr_t) &resp, tv) != RPC_SUCCESS)
+		     (caddr_t) &req, (xdrproc_t) xdr_void, NULL,
+		     tv) != RPC_SUCCESS)
 	{
 	  clnt_perror (PushClient, "YPPUSH: Cannot call YPPROC_XFR");
 	  kill (transid, SIGTERM);
 	}
 
-      /* XXX Check resp.xfrstat here ! */
-
 #if HAVE_WAIT4
       wait4 (transid, &sock, 0, &r);
 #else
