@@ -1,31 +1,25 @@
-/*
-** Copyright (c) 1996, 1997, 1998 Thorsten Kukuk
-**
-** This file is part of the NYS YP Server.
-**
-** The NYS YP Server is free software; you can redistribute it and/or
-** modify it under the terms of the GNU General Public License as
-** published by the Free Software Foundation; either version 2 of the
-** License, or (at your option) any later version.
-**
-** The NYS YP Server is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-** General Public License for more details.
-**
-** You should have received a copy of the GNU General Public
-** License along with the NYS YP Server; see the file COPYING.  If
-** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-** Cambridge, MA 02139, USA.
-**
-** Author: Thorsten Kukuk <kukuk@uni-paderborn.de>
-*/
+/* Copyright (c) 1996, 1997, 1998, 1999, 2000 Thorsten Kukuk
+   This file is part of the NYS YP Server.
+   Author: Thorsten Kukuk <kukuk@suse.de>
+   
+   The NYS YP Server is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
+   
+   The NYS YP Server is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+   
+   You should have received a copy of the GNU General Public
+   License along with the NYS YP Server; see the file COPYING.  If
+   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
+   Cambridge, MA 02139, USA. */
 
-#ifndef LINT
-static const char rcsid[] = "$Id: ypxfr.c,v 1.19 1998/07/02 06:29:59 kukuk Exp $";
-#endif
+#define _GNU_SOURCE
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
@@ -100,7 +94,7 @@ static DBM *dbm;
 #define YPMAPDIR "/var/yp"
 #endif
 
-#ifdef __NetBSD__
+#if defined (__NetBSD__) || (defined(__GLIBC__) && (__GLIBC__ == 2 && __GLIBC_MINOR__ == 0))
 /* <rpc/rpc.h> is missing the prototype */
 int callrpc (char *host, u_long prognum, u_long versnum, u_long procnum,
 	     xdrproc_t inproc, char *in, xdrproc_t outproc, char *out);
@@ -159,11 +153,11 @@ ypxfr_err_string (enum ypxfrstat error)
 #if defined(__NetBSD__)
 static int
 ypxfr_foreach (u_long status, char *key, int keylen,
-	       char *val, int vallen, void *data)
+	       char *val, int vallen, void *data __attribute__ ((unused)))
 #else
 static int
 ypxfr_foreach (int status, char *key, int keylen,
-	       char *val, int vallen, int *data)
+	       char *val, int vallen, char *data __attribute__ ((unused)))
 #endif
 {
   datum outKey, outData;
@@ -234,10 +228,8 @@ eachresult (bool_t * resp, struct sockaddr_in *addr)
     }
 }
 
-/*
-** At first, use the yp lib routines. If we are not bound to a server,
-** do a broadcast and search one.
-*/
+/* At first, use the yp lib routines. If we are not bound to a server,
+   do a broadcast and search one. */
 static int
 __yp_master (char *domain, char *map, char **master)
 {
@@ -282,6 +274,8 @@ __yp_master (char *domain, char *map, char **master)
   return status;
 }
 
+extern struct ypall_callback *xdr_ypall_callback;
+
 static enum ypxfrstat
 ypxfr (char *mapName)
 {
@@ -299,17 +293,15 @@ ypxfr (char *mapName)
   struct ypresp_master *resp_master;
   struct ypresp_val *resp_val;
   struct ypresp_all *resp_all;
-  extern struct ypall_callback *xdr_ypall_callback;
   datum outKey, outData;
   char orderNum[12];
   int result;
   int sock;
-  struct timeval TIMEOUT =
-  {25, 0};
+  struct timeval TIMEOUT = {25, 0};
 
   /*
-     ** Get the Master hostname for the map, if no explicit
-     ** sourcename is given.
+  ** Get the Master hostname for the map, if no explicit
+  ** sourcename is given.
    */
   if (SourceHost[0] == '\0')
     {
@@ -319,8 +311,8 @@ ypxfr (char *mapName)
     }
 
   /*
-     ** Build a connection to the host with the master map, not the
-     ** the nearest ypserv, because this map could be out of date.
+  ** Build a connection to the host with the master map, not the
+  ** the nearest ypserv, because this map could be out of date.
    */
   memset (&sockaddr, '\0', sizeof (sockaddr));
   sockaddr.sin_family = AF_INET;
@@ -354,7 +346,9 @@ ypxfr (char *mapName)
   if ((resp_order = ypproc_order_2 (&req_nokey, clnt_udp)) == NULL)
     {
       clnt_perror (clnt_udp, "masterOrderNum");
-      masterOrderNum = 0x7fffffff;
+      masterOrderNum = time (NULL); /* We set it to the current time.
+                                       So a new map will be always newer. */
+
     }
   else
     {
@@ -376,9 +370,7 @@ ypxfr (char *mapName)
 	  xdr_free ((xdrproc_t) xdr_ypresp_master, (char *) resp_master);
 	}
     }
-  /*
-     ** If we doesn't force the map, look, if the new map is really newer
-   */
+  /* If we doesn't force the map, look, if the new map is really newer */
   if (!Force)
     {
       datum inKey, inVal;
@@ -438,9 +430,9 @@ ypxfr (char *mapName)
     }
 
   /*
-     ** Try to use ypxfrd for getting the new map. If it fails, use the old
-     ** method.
-   */
+  ** Try to use ypxfrd for getting the new map. If it fails, use the old
+  ** method.
+  */
   if ((result = ypxfrd_transfer (masterName, mapName, TargetDomain, dbName)) != 0)
     {
 
@@ -573,8 +565,14 @@ ypxfr (char *mapName)
       yp_msg ("ERROR: Path to long: %s/%s/%s\n", path_ypdb, TargetDomain, mapName);
       return YPXFR_RSRC;
     }
-  unlink (dbName2);
-  rename (dbName, dbName2);
+
+  if (result == 0)
+    {
+      unlink (dbName2);
+      rename (dbName, dbName2);
+    }
+  else
+    unlink(dbName);
 
   if (!NoClear)
     {
@@ -594,7 +592,7 @@ ypxfr (char *mapName)
   return result == 0 ? YPXFR_SUCC : YPXFR_YPERR;
 }
 
-void
+static void
 Usage (int exit_code)
 {
   fprintf (stderr, "usage: ypxfr [-f] [-c] [-d target domain] [-h source host] [-s source domain]\n");
